#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.Xml;

namespace HeuristicLab.Clients.OKB {
  [View("OKB Query")]
  [Content(typeof(OKBClient), false)]
  public sealed partial class QueryView : HeuristicLab.MainForm.WindowsForms.View {
    private CombinedFilterView combinedFilterView;

    public QueryView() {
      InitializeComponent();
    }

    protected override void OnInitialized(EventArgs e) {
      base.OnInitialized(e);
      this.Cursor = Cursors.AppStarting;

      Func<CombinedFilter> func = () => {
        return OKBClient.Instance.GetFilters().OfType<CombinedFilter>().Where(x => x.Operation == BooleanOperation.And).FirstOrDefault();
      };

      func.BeginInvoke((IAsyncResult result) => {
        CombinedFilter filter = func.EndInvoke(result);
        if (InvokeRequired) Invoke(new Action<CombinedFilter>(FinishedLoadingFilter), filter);
        else FinishedLoadingFilter(filter);
      }, null);
    }

    private void FinishedLoadingFilter(CombinedFilter filter) {
      if (filter != null) {
        combinedFilterView = (CombinedFilterView)MainFormManager.CreateView(typeof(CombinedFilterView));
        combinedFilterView.Content = (CombinedFilter)filter.Clone();
        Control control = (Control)combinedFilterView;
        control.Dock = DockStyle.Fill;
        filterPanel.Controls.Add(control);
      }
      this.Cursor = Cursors.Default;
      SetEnabledStateOfControls();
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      resultsGroupBox.Enabled = combinedFilterView != null;
    }

    private void refreshResultsButton_Click(object sender, EventArgs e) {
      bool deserialize = deserializeBlobsCheckBox.Checked;

      System.Diagnostics.Stopwatch sw = new System.Diagnostics.Stopwatch();
      sw.Start();
      long count = OKBClient.Instance.GetNumberOfQueryResults(combinedFilterView.Content);
      sw.Stop();
      MessageBox.Show(count.ToString() + " Runs");
      if (MessageBox.Show(sw.Elapsed.ToString(), "GetNumberOfQueryResults", MessageBoxButtons.OKCancel) == DialogResult.Cancel) return;

      sw.Restart();
      var ids = OKBClient.Instance.GetQueryResultIds(combinedFilterView.Content);
      sw.Stop();
      if (MessageBox.Show(sw.Elapsed.ToString(), "GetQueryResultIds", MessageBoxButtons.OKCancel) == DialogResult.Cancel) return;

      sw.Restart();
      List<QueryResult> results = new List<QueryResult>();
      while (ids.Count() > 0) {
        results.AddRange(OKBClient.Instance.GetQueryResults(ids.Take(50)));
        ids = ids.Skip(50);
      }
      sw.Stop();
      if (MessageBox.Show(sw.Elapsed.ToString(), "GetQueryResults", MessageBoxButtons.OKCancel) == DialogResult.Cancel) return;

      sw.Restart();
      IEnumerable<QueryResult> res = results.AsEnumerable();
      RunCollection runs = new RunCollection();
      runCollectionView.Content = runs;
      while (res.Count() > 0) {
        runs.AddRange(res.Take(50).Select(x => ConvertToOptimizationRun(x, deserialize)));
        res = res.Skip(50);
      }
      sw.Stop();
      MessageBox.Show(sw.Elapsed.ToString(), "Conversion", MessageBoxButtons.OK);

      SetEnabledStateOfControls();
    }

    private Optimization.IRun ConvertToOptimizationRun(QueryResult queryResult, bool deserialize) {
      Optimization.Run run = new Optimization.Run();
      foreach (QueryValue value in queryResult.AlgorithmParameters)
        run.Parameters.Add(value.Name, ConvertToItem(value, deserialize));
      foreach (QueryValue value in queryResult.ProblemParameters)
        run.Parameters.Add(value.Name, ConvertToItem(value, deserialize));
      foreach (QueryValue value in queryResult.Results)
        run.Results.Add(value.Name, ConvertToItem(value, deserialize));
      return run;
    }

    private IItem ConvertToItem(QueryValue value, bool deserialize) {
      if (value is QueryBlobValue) {
        if (deserialize) {
          IItem item = null;
          using (MemoryStream stream = new MemoryStream(((QueryBlobValue)value).Value)) {
            try {
              item = XmlParser.Deserialize<IItem>(stream);
            }
            catch (Exception) { }
            stream.Close();
          }
          return item != null ? item : new StringValue(((QueryBlobValue)value).DataTypeName);
        } else {
          return new StringValue(((QueryBlobValue)value).DataTypeName);
        }
      } else if (value is QueryBoolValue) {
        return new BoolValue(((QueryBoolValue)value).Value);
      } else if (value is QueryFloatValue) {
        return new DoubleValue(((QueryFloatValue)value).Value);
      } else if (value is QueryIntValue) {
        return new IntValue((int)((QueryIntValue)value).Value);
      } else if (value is QueryStringValue) {
        return new StringValue(((QueryStringValue)value).Value);
      }
      return null;
    }
  }
}
